@echo off 

rem last update on January 28, 2016
rem spelling error fixed for "coastbeach0?_n.dds"
rem update on June 13, 2015
rem this update adds a Vanilla Compressed Textures temporary folder for DirectX compressed terrain data from Skyrim - textures.bsa as it needs to use the DXTx Constraint tab parameters
rem This is only added if the ''Vanilla Extracted\STD'' folder is present
rem update on January 18. 2015
rem this was a small update that excluded the uncompressed "HRDLC2\textures\terrain\tamriel.objects.dds" from being optimized

rem Batch file to perform several tasks used to prepare vanilla textures for optimization using DDSopt
rem It can be used with extracted textures from one or more of the vanilla BSAs
rem It does not require that the extracted textures from all of the vanilla BSAs are present
rem as per steps in the DDSopt guide (http://wiki.step-project.com/Guide:DDSopt#tab=DDSopt_Optimization).
rem It is only useful when being used with other steps described in this guide.
rem If this batch file is edited, this MUST BE DONE with an editor that does not add additional characters (e.g., line feed, carriage return)
rem these characters are often invisible but can sometimes cause the batch file to malfunction
rem DO NOT USE a default Windows text editor to edit this file
rem Editors like Notepad++ (which was used to create this file), TextPad, UltraEdit are fine
 
rem It expects that there is a Working directory containing the "Vanilla Extracted" directory
rem This directory should not be in any User Access Controlled (UAC) directory such as Program Files or Program Files (x86).
rem It expects that the BSA contents have already been extracted into folders in the  "Vanilla Extracted" directory
rem It expects a folder in "Vanilla Extracted" labeled "STD"
rem If the HRDLC are being optimized, the extracted BSA contents should be in "HRDLC1", "HRDLC2", "HRDLC3" in "Vanilla Extracted"
rem If there are any optional DLC being used their BSAs should have also been extracted into "Vanilla Extracted"
rem with folder names of "Dawnguard", "Hearthfires", and "Dragonborn" (based on the DLC being used).
rem
rem The batch file copies interface files in the STD and HRDLC folders in "Vanilla Extracted" to "Vanilla Optimized"
rem The interface files in the DLC BSAs remain there so they don't need to be copied
rem The batch file then copies 14 compressed, smooth normal maps to Vanilla Optimized (added 9/12/2013)
rem  to avoid optimizing them with DDSopt them as the optimization exarbates some vanilla artifacts
rem
rem  The batch file sorts the textures in "Vanilla Extracted" into five folders
rem  These are "Vanilla Ordinary Textures" for the non-exterior and non-body-related color maps,
rem  "Vanilla Normal Maps" for the non-exterior and non-body-related tangent space normal maps
rem  "Vanilla Body Textures" for the body-related textures
rem  "Vanilla Exterior Textures" for the color maps from the exterior texture directories (\architecture, \landscape, and \terrain)
rem  "Vanilla Exterior Normal Maps" for the terrain space normal maps in the exterior texture directories

rem  There are a few exceptions to this
rem  A few files are moved directly to the output, "Vanilla Optimized" so they are not optimized by DDSopt.
rem  In addition to those already mentioned above this includes some HRDLC3 model space normal maps
rem  It also includes all the small facial textures in the \tintmask, \facetint, and \facemods folders
rem  These were added because the Creation Kit, and presumably other texture creation software, needs the uncompressed versions of these
rem  The uncompressed terrain folder normal maps in HRDLC are put in "Vanilla Exterior Normal Maps"
rem  However, the rest of the terrain folder normal maps are put in "Vanilla Exterior Textures"
rem  This is done so they can be optimized as DirectX textures
rem  (vs. using the uncompressed format DDSopt parameters used for the other folders)
rem  most of these are monocolor textures that DDSopt will collapse into 1x1 textures, saving space
rem
rem While the batch file runs there will be a command window that echoes the commands
rem This allows monitoring the progress of the batch processing if desired, although it is not required.
 
rem the initial 2 lines should ensure that the system provides the correct working directory name in %cd%
rem This is needed for a small percentage of systems, probably because of interaction with Windows UAC on those systems

REM tests added 8/18/2013 to make sure the batch files are in the right folder and the folder structures are correct
rem 2/4/2014 test added for presence of windows\system32\robocopy.exe




IF NOT EXIST "%Windir%\system32\robocopy.exe" echo ERROR: The Robocopy system program isn't available; fix this before proceeding

@setlocal enableextensions
@cd /d "%~dp0"
 
echo.
echo.
echo The system says the Working Directory is   %cd%
echo This directory should include the "Vanilla Extracted" directory
echo If any of the above is incorrect terminate the processing and fix the problem
echo.

rem test added 12/19/2013 and modified 1/13/2014 to test for administrative privileges
rem taken from http://stackoverflow.com/questions/4051883/batch-script-how-to-check-for-admin-rights
rem administrative privilege isn't usually needed
:testPrivilegies
    set guid=%random%%random%-%random%-%random%-%random%-%random%%random%%random%
    mkdir %WINDIR%\%guid%>nul 2>&1
    rmdir %WINDIR%\%guid%>nul 2>&1
    IF NOT %ERRORLEVEL%==0 echo  Administrative privileges are not enabled; this is usually not a problem
        


IF NOT EXIST "%cd%\Vanilla Extracted" echo  Error: The Vanilla Extracted directory is not in the working directory; fix this before proceeding
echo.
echo. If the HRDLC are used then the DLC BSAs do not need to be optimized; the HRDLC iBSAs nclude all the textures from the DLC that should be optimized
echo  ---------------------
echo On some systems the batch file may take a long time (30 minutes or so) and might not seem to be working
echo If it starts successfully and gets through a step or two it is almost certainly working so be patient with it
echo it's doing the best it can
echo ---------------------
echo Hit enter to continue after the pause
pause


 
IF NOT EXIST "%cd%\Vanilla Ordinary Textures\NUL"  MD "Vanilla Ordinary Textures"
IF NOT EXIST "%cd%\Vanilla Normal Maps\NUL"  MD "Vanilla Normal Maps"
IF NOT EXIST "%cd%\Vanilla Exterior Textures\NUL"  MD "Vanilla Exterior Textures"
IF NOT EXIST "%cd%\Vanilla Exterior Normal Maps\NUL"  MD "Vanilla Exterior Normal Maps"
IF NOT EXIST "%cd%\Vanilla Optimized\NUL"  MD "Vanilla Optimized"
IF NOT EXIST "%cd%\Vanilla Body Textures\NUL"  MD "Vanilla Body Textures"
IF EXIST "%cd%\Vanilla Extracted\STD"  IF NOT EXIST "%cd%\Vanilla Compressed\NUL"  MD "Vanilla Compressed"


REM Get the starting time
set STARTTIME=%TIME%


@echo on

rem If the STD and/or the HRDLC2 folders are present prepare to copy the interface and facegen  textures to "Vanilla Optimized" as they do not need optimization
cd "Vanilla Optimized"
IF EXIST "..\Vanilla Extracted\STD" IF NOT EXIST \STD\NUL  MD STD
IF EXIST "..\Vanilla Extracted\STD" IF NOT EXIST \STD\textures\interface\NUL  MD STD\textures\interface\
IF EXIST "..\Vanilla Extracted\STD" IF NOT EXIST \STD\textures\actors\character\facegendata\NUL  MD STD\textures\actors\character\facegendata\
IF EXIST "..\Vanilla Extracted\HRDLC2" IF NOT EXIST \HRDLC2\NUL  MD STD
IF EXIST "..\Vanilla Extracted\HRDLC2" IF NOT EXIST \HRDLC2\textures\interface\NUL  MD HRDLC2\textures\interface\
cd ..

IF NOT EXIST "%cd%Vanilla Compressed" goto start
cd "Vanilla Compressed"
IF EXIST "..\Vanilla Extracted\STD" IF NOT EXIST \STD\textures\terrain\NUL MD STD\textures\terrain\
cd ..


:start
rem move some HRDLC interface textures to "Vanilla Optimized"
IF EXIST "%cd%\Vanilla Extracted\HRDLC2" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC2\textures\interface" "%cd%\Vanilla Optimized\HRDLC2\textures\interface" *.* /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem copy some compressed, smooth normal maps to Vanilla Optimized to avoid optimizing them as the optimization makes some  vanilla artifacts even worse
IF EXIST "%cd%\Vanilla Extracted\STD" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\STD" "%cd%\Vanilla Optimized\STD" coastbeach0?_n.dds dirt0?_n.dds dirtpath0?_n.dds dirtsnowpath0?_n.dds snow0?_n.dds tundra0?_n.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt


rem copy all non-interface and non-facegendata textures to Vanilla Body Textures, excluding files that have problems when optimized
%Windir%\system32\robocopy "%cd%\Vanilla Extracted" "%cd%\Vanilla Body Textures" *.dds /s /xf coastbeach0?_n.dds dirt0?_n.dds dirtpath0?_n.dds dirtsnowpath0?_n.dds snow0?_n.dds tundra0?_n.dds /xd *interface  *facegendata /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem Move HRDLC3 MSN texture files to "Vanilla Optimized" to ensure they are unchanged
rem If resolution of the 2Kx2K MSN is reduced it will be corrupted (although optimized with R5G6B5 format will work fine)
IF EXIST "%cd%\Vanilla Extracted\HRDLC3" %Windir%\system32\robocopy "%cd%\Vanilla Body Textures\HRDLC3" "%cd%\Vanilla Optimized\HRDLC3" *_msn.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem Move one uncompressed terrain texture in HRDLC2  to "Vanilla Optimized" so it won't be optimized (and corrupted)
IF EXIST "%cd%\Vanilla Extracted\HRDLC2" %Windir%\system32\robocopy "%cd%\Vanilla Body Textures\HRDLC2" "%cd%\Vanilla Optimized\HRDLC2" tamriel.objects.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move all textures to Vanilla Ordinary Textures except those that aren't being optimized, namely textures in \tintmasks and \facemods folders.
%Windir%\system32\robocopy "%cd%\Vanilla Body Textures" "%cd%\Vanilla Ordinary Textures" *.dds /s  /move /xd *tintmasks *facetint *facemods /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move all the remaining textures in "Vanilla Ordinary Textures" to "Vanilla Optimized", namely tintmasks and facemods, to "Vanilla Optimized" so they are not optimized
%Windir%\system32\robocopy "%cd%\Vanilla Body Textures" "%cd%\Vanilla Optimized" *.dds /s  /move  /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem "Vanilla Body Textures" is temporarily empty

rem move the DreectX compressed terrain textures in STD to Vanilla Compressed Textures
IF EXIST "%cd%\Vanilla Extracted\STD" %Windir%\system32\robocopy "%cd%\Vanilla Ordinary Textures\STD\textures\terrain" "%cd%\Vanilla Compressed Textures\STD\textures\terrain" *.dds /s  /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move all non-exterior textures to Vanilla Body Textures
%Windir%\system32\robocopy "%cd%\Vanilla Ordinary Textures" "%cd%\Vanilla Body Textures" *.dds /s  /move /xd *architecture *landscape *terrain /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Vanilla Ordinary Textures temporarily has only exterior textures

rem move all exterior normal map textures to Vanilla Exterior Normal Maps
%Windir%\system32\robocopy "%cd%\Vanilla Ordinary Textures" "%cd%\Vanilla Exterior Normal Maps" *_n.dds /s  /move /xd  /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move the all exterior color map textures to Vanilla Exterior Textures
%Windir%\system32\robocopy "%cd%\Vanilla Ordinary Textures" "%cd%\Vanilla Exterior Textures" *.dds /s /move /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Vanilla Ordinary Textures is now temporarily empty

rem move all non-body-related normal map textures to Vanilla Normal Maps
%Windir%\system32\robocopy "%cd%\Vanilla Body Textures" "%cd%\Vanilla Normal Maps" *_n.dds /s  /move /xd *character /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem move all remaining non-body-related textures to Vanilla Ordinary Textures
%Windir%\system32\robocopy "%cd%\Vanilla Body Textures" "%cd%\Vanilla Ordinary Textures" *.dds /s  /move /xf *_msn.dds /xd *character /nfl /ndl /njh /njs /ns /nc /np >>log.txt


rem add some textures from STD that don't get optimized
IF EXIST "%cd%\Vanilla Extracted\STD" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\STD\textures\interface" "%cd%\Vanilla Optimized\STD\textures\interface" *.* /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
IF EXIST "%cd%\Vanilla Extracted\STD" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\STD\textures\actors\character\facegendata" "%cd%\Vanilla Optimized\STD\textures\actors\character\facegendata" *.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem my work here is done

@echo off


rem get time of completion
set ENDTIME=%TIME%

rem from http://stackoverflow.com/questions/9922498/calculate-time-difference-in-batch-file

rem Change formatting for the start and end times
for /F "tokens=1-4 delims=:.," %%a in ("%STARTTIME%") do (
    set /A "start=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

for /F "tokens=1-4 delims=:.," %%a in ("%ENDTIME%") do (
    set /A "end=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

rem Calculate the elapsed time by subtracting values
set /A elapsed=end-start

rem Format the results for output
set /A hh=elapsed/(60*60*100), rest=elapsed%%(60*60*100), mm=rest/(60*100), rest%%=60*100, ss=rest/100, cc=rest%%100
if %hh% lss 10 set hh=0%hh%
if %mm% lss 10 set mm=0%mm%
if %ss% lss 10 set ss=0%ss%
if %cc% lss 10 set cc=0%cc%

set DURATION=%hh%:%mm%:%ss%,%cc%


echo.
echo.
echo Start    : %STARTTIME%
echo Finish   : %ENDTIME%
echo          ---------------
echo Duration : %DURATION%

echo.
echo The batch file processing has finished.
echo  Check the log file (log.txt). The log file should not show any errors and should have 15 blank lines.
echo.
pause

